
// dllMain.cpp : Defines the exported functions for the DLL application.

#include "ThereminoSlots.h"

#include <string.h>

// Headers needed for sleeping.
#ifdef _WIN32
	#include <windows.h>
#else
	#include <unistd.h>
#endif

#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/types.h> //shm_open
#include <stdio.h>  //printf
#include <stdlib.h> //exit
#include <unistd.h> //close
#include <string.h> //strerror

/* This will be created under /dev/shm/ */
#define SHARED_FILE "/theremino1.shared"

/* Define a struct we wish to share. Notice that we will allocate
 * only sizeof SHARED_VAR, so all sizes are constant
 */
typedef struct
{
	float slots[1000];
} SHARED_VAR;

// ===========================================================================================
//   GLOBALS
// ===========================================================================================
int first = 0;
int result = 0;
int initialized = 0;
int shm_fd;
static SHARED_VAR *pool;

// ===========================================================================================
//   EXPORTED FUNCTIONS
//
//  The normal CallingConvention ( Declare in VB ) is Winapi, which defaults to StdCall
// ===========================================================================================


int the_slot_init()
{
	/* Try to open the shm instance with  O_EXCL,
	* this tests if the shm is already opened by someone else
	*/
	if ((shm_fd = shm_open(SHARED_FILE, (O_CREAT | O_EXCL | O_RDWR),
						   (S_IREAD | S_IWRITE))) > 0 )
	{
		first = 1; /* We are the first instance */
	}
	else if ((shm_fd = shm_open(SHARED_FILE, (O_CREAT | O_RDWR),
								(S_IREAD | S_IWRITE))) < 0)
	{
		/* Try to open the shm instance normally and share it with
		* existing clients
		*/
		//printf("Could not create shm object. %s\n", strerror(errno));
		return -1;
	}

	/* Set the size of the SHM to be the size of the struct. */
	result = ftruncate(shm_fd, sizeof(SHARED_VAR));

	/* Connect the pool pointer to set to the shared memory area,
	* with desired permissions
	*/
	if((pool =  mmap(0, sizeof(SHARED_VAR), (PROT_READ | PROT_WRITE),
					 MAP_SHARED, shm_fd, 0)) == MAP_FAILED)
	{
		return -2;
	}

	initialized = 1;
	return 0;
}

int the_slot_exit()
{
	close(shm_fd);
	return 0;
}




//float slots[1000];

short IsValidSlot(int slot)
{
	if (slot < 0) return 0;
	if (slot > 999) return 0;
	return 1;
}

float the_slot_read(int slot)
{
	if (!initialized) return 0.0F;

	if (IsValidSlot(slot))
		return pool->slots[slot];
	else
		return 0.0F;
}

void the_slot_write(int slot, float value)
{
	if (!initialized) return;

	if (IsValidSlot(slot))
		pool->slots[slot] = value;
}

